<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Pinpoint\CarouselManager\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\CarouselManager\Api\CarouselRepositoryInterface;
use Pinpoint\CarouselManager\Api\Data\CarouselInterfaceFactory;
use Pinpoint\CarouselManager\Api\Data\CarouselSearchResultsInterfaceFactory;
use Pinpoint\CarouselManager\Model\ResourceModel\Carousel as ResourceCarousel;
use Pinpoint\CarouselManager\Model\ResourceModel\Carousel\CollectionFactory as CarouselCollectionFactory;

class CarouselRepository implements CarouselRepositoryInterface
{

    protected $resource;

    protected $carouselFactory;

    protected $carouselCollectionFactory;

    protected $searchResultsFactory;

    protected $dataObjectHelper;

    protected $dataObjectProcessor;

    protected $dataCarouselFactory;

    protected $extensionAttributesJoinProcessor;

    private $storeManager;

    private $collectionProcessor;

    protected $extensibleDataObjectConverter;

    /**
     * @param ResourceCarousel $resource
     * @param CarouselFactory $carouselFactory
     * @param CarouselInterfaceFactory $dataCarouselFactory
     * @param CarouselCollectionFactory $carouselCollectionFactory
     * @param CarouselSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     * @param ExtensibleDataObjectConverter $extensibleDataObjectConverter
     */
    public function __construct(
        ResourceCarousel $resource,
        CarouselFactory $carouselFactory,
        CarouselInterfaceFactory $dataCarouselFactory,
        CarouselCollectionFactory $carouselCollectionFactory,
        CarouselSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource = $resource;
        $this->carouselFactory = $carouselFactory;
        $this->carouselCollectionFactory = $carouselCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataCarouselFactory = $dataCarouselFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
        $this->collectionProcessor = $collectionProcessor;
        $this->extensionAttributesJoinProcessor = $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        \Pinpoint\CarouselManager\Api\Data\CarouselInterface $carousel
    ) {

        $carouselData = $this->extensibleDataObjectConverter->toNestedArray(
            $carousel,
            [],
            \Pinpoint\CarouselManager\Api\Data\CarouselInterface::class
        );
        
        $carouselModel = $this->carouselFactory->create()->setData($carouselData);
        
        try {
            $this->resource->save($carouselModel);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the carousel: %1',
                $exception->getMessage()
            ));
        }
        return $carouselModel->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function get($carouselId)
    {
        $carousel = $this->carouselFactory->create();
        $this->resource->load($carousel, $carouselId);
        if (!$carousel->getId()) {
            throw new NoSuchEntityException(__('Carousel with id "%1" does not exist.', $carouselId));
        }
        return $carousel->getDataModel();
    }

    /**
     * @param $carouselId
     * @return mixed
     * @throws NoSuchEntityException
     */
    public function getById($carouselId)
    {
        $carousel = $this->carouselFactory->create();
        if (is_numeric($carouselId)) {
            $this->resource->load($carousel, $carouselId);
        } else if (is_string($carouselId)) {
            $this->resource->load($carousel, $carouselId, 'identifier');
        }

        if (!$carousel->getId()) {
            throw new NoSuchEntityException(__('Carousel with id "%1" does not exist.', $carouselId));
        }
        return $carousel;
    }


    public function getOptionArray()
    {
        return $this->carouselCollectionFactory->create()->toOptionArray();
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->carouselCollectionFactory->create();
        
        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \Pinpoint\CarouselManager\Api\Data\CarouselInterface::class
        );
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        \Pinpoint\CarouselManager\Api\Data\CarouselInterface $carousel
    ) {
        try {
            $carouselModel = $this->carouselFactory->create();
            $this->resource->load($carouselModel, $carousel->getCarouselId());
            $this->resource->delete($carouselModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the Carousel: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($carouselId)
    {
        return $this->delete($this->get($carouselId));
    }
}

